(*+--------------------------------------------------------------------------+
  |                                                                          |
  |                  Calcul d'images de Julia et Mandelbrot                  |
  |                                                                          |
  +--------------------------------------------------------------------------+*)

open Printf
open Parallele
open Global
open Graphics


                 (* +-----------------------------+
                    |  Impression pour dboguage  |
                    +-----------------------------+ *)

let soi() = ""
let soj(d,bx,by) = sprintf "%s %d %d"
		     (match d.nat with
			| MANDELBROT -> "MANDELBROT"
			| JULIA      -> "JULIA")
		     bx by
let sor(r) = match r with [] -> "" | (x,y,_)::_ -> sprintf "%d %d" x y


                      (* +-------------------+
                         |  calcule un bloc  |
                         +-------------------+ *)
(*
let pixel(x,y,jx,jy,nmax,psize) =
  let u  = ref(x)    and v  = ref(y)    in
  let u2 = ref(x*.x) and v2 = ref(y*.y) in
  let n  = ref(0) in
  while !u2 +. !v2 < 4.0 && !n < nmax do
    v  := 2.0*.(!u)*.(!v) +. jy;
    u  := !u2 -. !v2 +. jx;
    u2 := !u*.(!u);
    v2 := !v*.(!v);
    incr(n)
  done;
  (!n *(psize-1))/nmax
*)

(* cod en C, voir pixel.c *)
external pixel : (float * float * float * float * int * int) -> int = "pixel"

let calc (d,bx,by) =
  let res = ref([]) in
  for y = (min (by + d.hy) d.sy) - 1 downto by do
    let l = ref [] in
    for x = (min (bx + d.hx) d.sx) - 1 downto bx do
      let u = d.x0 -. d.dx/.2.0 +. float(x)*.d.dx/.float(d.sx)
      and v = d.y0 -. d.dy/.2.0 +. float(y)*.d.dy/.float(d.sy) in
      let n = match d.nat with
	| JULIA      -> pixel(u,v,     d.jx,d.jy,  d.nmax,d.psize)
	| MANDELBROT -> pixel(0.0,0.0, u,v,        d.nmax,d.psize)
      in
      match !l with
	| (c,v)::suite when v = n -> l := (c+1,v)::suite
	| _ -> l := (1,n) :: !l
    done;
    res := (bx,y,!l) :: !res
  done;
  !res


                   (* +-------------------------+
                      |  Donne un nouveau bloc  |
                      +-------------------------+ *)


let job data () =
  match data.tat with
    | Computing(img) when data.by < img.sy ->
	let j = (img,data.bx,data.by) in
	data.bx <- data.bx + img.hx;
	if data.bx >= img.sx then (data.bx <- 0; data.by <- data.by + img.hy);
	j
    | _ -> raise Job_wait
	

                      (* +------------------+
                         |  Reoit un bloc  |
                         +------------------+ *)


let rec affiche_ligne x y = function
  | [] -> ()
  | (c,v)::suite ->
      set_color palette.(v);
      lineto (x+c-1) y;
      affiche_ligne (x+c) y suite
      
let rec affiche_bloc fx fy = function
  | [] -> ()
  | (x,y,l)::suite ->
      let x0 = x + fx and y0 = y + fy in
      moveto x0 y0;
      affiche_ligne x0 y0 l;
      affiche_bloc fx fy suite

let result data _ bloc reste =
  affiche_bloc data.fdraw.fx data.fdraw.fy bloc;
  match Widgets.vnement true {entres=[]; boutons=data.bctrl; onglets=[]} with
    | Some(BOUTON "stop") -> data.tat <- Interrupted; reste
    | Some(TOUCHE '\027') -> data.tat <- Interrupted; reste
    | _ -> []

                 (* +-----------------------------+
                    |  Gestion du cache d'images  |
                    +-----------------------------+ *)

let insre_image data img pixmap =
  
  let rec delete n1 n2 = function
    | [] when n1+n2 < data.maxmem -> (false,false,[])
    | [] -> (n2 < 2, n2 >= 2, [])
    | i::suite when (fst i).nat = img.nat ->
	let (d,d',reste) = delete (n1+1) n2 suite in
	if d then (false,false,reste) else (false,d',i::reste)
    | i::suite ->
	let (d,d',reste) = delete n1 (n2+1) suite in
	if d' then (false,false,reste) else (d,false,i::reste)
  in
  let (_,_,reste) = delete 0 0 data.mem in
  data.mem <- (img,pixmap) :: reste


       (* +-------------------------------------------------+
          |  Extrait une image du cache ou lance le calcul  |
          +-------------------------------------------------+ *)

let calc_image master data img =

  try
    let pixmap = List.assoc img data.mem in
    draw_image pixmap data.fdraw.fx data.fdraw.fy

  with Not_found ->
    data.tat <- Computing(img);
    data.bx   <- 0;
    data.by   <- 0;
    let pixmap = Widgets.get_image data.fmsg in
    data.bctrl <- [Widgets.bouton_texte(data.fmsg,"stop")];
    List.iter (Widgets.draw_bouton false) data.bctrl;
    Widgets.fill_fentre data.fdraw fond_actif;
    Master.start master;
    draw_image pixmap data.fmsg.fx data.fmsg.fy;
    if data.tat <> Interrupted then 
      let pixmap = Widgets.get_image data.fdraw in
      insre_image data img pixmap


