(* +------------------------------------------------------------------------+
   |                                                                        |
   |                          Dfinitions globales                          |
   |                                                                        |
   +------------------------------------------------------------------------+ *)

                          (* +---------------------+
                             |  Aspect de l'cran  |
                             +---------------------+ *)


(* palette de couleurs pour les fractals *)
let palette = [|
  0xffffff ; 0xfff7ee ; 0xffeedd ; 0xffe6cc ;
  0xffddbb ; 0xffd5aa ; 0xffcc99 ; 0xffc488 ;
  0xffbb77 ; 0xffb366 ; 0xffaa55 ; 0xffa244 ;
  0xff9933 ; 0xff9122 ; 0xff8811 ; 0xff8000 ;
  0xee8811 ; 0xdd9122 ; 0xcc9933 ; 0xbba244 ;
  0xaaaa55 ; 0x99b366 ; 0x88bb77 ; 0x77c488 ;
  0x66cc99 ; 0x55d5aa ; 0x44ddbb ; 0x33e6cc ;
  0x22eedd ; 0x11f7ee ; 0x00ffff ; 0x000000 
|]

(* couleurs des lments graphiques *)
let fond_actif    = 0xd8b8a8
and fond_passif   = 0xc8a898
and fond_gris     = 0xc0c0c0
and clair         = 0xffffff
and sombre        = 0x000000
and crayon        = 0x000000

(* fonte texte *)
let text_font   = "10x20"

(* dimensions *)
let minwidth = 600   (* largeur minimum    *)
and minheight= 415   (* hauteur minimum    *)
and pad      = 5     (* distance aux bords *)
and fl_width = 20    (* taille flches     *)
and oheight  = 25    (* hauteur onglets    *)
and owidth   = 120   (* largeur onglets    *)
and bheight  = 25    (* hauteur boutons    *)
and en_width = 25    (* largeur entres    *)

(* vnements reconnus *)
type vnement =
  | FLECHE of int
  | BOUTON of string
  | ONGLET of string
  | TOUCHE of char
  | POINT  of int * int
  | RESIZE

                          (* +-----------+
                             |  Widgets  |
                             +-----------+ *)

type fentre = {
  fx : int;        	    (* abscisse coin BG *)
  fy : int;        	    (* ordonne coin BG *)
  fw : int;        	    (* largeur          *)
  fh : int                  (* hauteur          *)
}

type bouton = { 
  bf : fentre;            (* contour          *)
  mutable clic : bool;     (* enfonc ?        *)
  bevt : vnement;        (* evt engendr     *)
  draw : fentre -> unit   (* dessin intrieur *)
}

type entre = {
  ef   : fentre;          (* contour     *)
  leg  : string;           (* lgende     *)
  mutable texte : string   (* texte dit *)
}

type onglet = {
  ow    : fentre;         (* contour *)
  titre : string           (* lgende *)
}

type widget_set = {
  boutons : bouton list;
  onglets : onglet list;
  entres : entre list
}

                  (* +---------------------------+
                     |  Description des calculs  |
                     +---------------------------+ *)

type image_nat = JULIA | MANDELBROT

type image = {                    (* description d'une image *)
  nat : image_nat;                (* nature de l'image       *)
  x0  : float; 	y0 : float; 	  (* coordonnes centre      *)
  dx  : float; 	dy : float; 	  (* dimensions virtuelles   *)
  sx  : int;   	sy : int;   	  (* nombre de pixels        *)
  jx  : float; 	jy : float; 	  (* paramtres de Julia     *)
  hx  : int;   	hy : int;  	  (* taille d'un bloc        *)
  nmax  : int;             	  (* nb maxi d'itrations    *)
  psize : int                     (* taille de la palette    *)
}

(* codage d'une image : liste de listes de segments horizontaux *)
type ligne = int * int * ((int * int) list)   (* (x,y,segments) *)
type bloc  = ligne list

type tat = Computing of image | Interrupted | Done


                    (* +----------------------+
                       |  Paramtres globaux  |
                       +----------------------+ *)

type data = {
  mutable julia  : image;                          (* descr. Julia        *)
  mutable mandel : image;                          (* descr. Mandelbrot   *)
  mutable mem    : (image * Graphics.image) list;  (* cache d'images      *)
  maxmem         : int;                            (* taille maxi cache   *)
  mutable ongl   : onglet list;                    (* onglets ordonns    *)
  mutable fongl  : fentre;                        (* zone onglets        *)
  mutable fdraw  : fentre;                        (* zone d'affichage    *)
  mutable fmsg   : fentre;                        (* zone de messages    *)
  mutable bctrl  : bouton list;                    (* boutons de contrle *)
  mutable tat 	 : tat;                  	   (* activit            *)
  mutable bx   	 : int;                   	   (* prochaine abscisse  *)
  mutable by   	 : int;                   	   (* prochaine ordonne  *)
  mutable fname  : string                          (* pour enregistrement *)
}

let wx = ref(0) (* largeur totale *)
and wy = ref(0) (* hauteur totale *)
