(* +------------------------------------------------------------------------+
   |                                                                        |
   |                        Edition des paramtres                          |
   |                                                                        |
   +------------------------------------------------------------------------+ *)

open Graphics
open Global
open Widgets

                        (* +-------------------------+
                           |  initialise une entre  |
                           +-------------------------+ *)

let set_champ img entre = match entre.leg with
  | "x0: "   ->	entre.texte <- string_of_float(img.x0)
  | "y0: "   ->	entre.texte <- string_of_float(img.y0)
  | "dx: "   ->	entre.texte <- string_of_float(img.dx)
  | "dy: "   ->	entre.texte <- string_of_float(img.dy)
  | "jx: "   ->	entre.texte <- string_of_float(img.jx)
  | "jy: "   ->	entre.texte <- string_of_float(img.jy)
  | "nmax: " ->	entre.texte <- string_of_int(img.nmax)
  | _ -> failwith "champ inconnu"

                          (* +--------------------+
                             |  relit une entre  |
                             +--------------------+ *)

let get_float v entre =
  let s = strip(entre) in
  if string_of_float(v) = s then v else float_of_string(s)


let valide img entre = try
  match entre.leg with
    | "x0: " -> {img with x0 = get_float img.x0 entre}
    | "y0: " -> {img with y0 = get_float img.y0 entre}
    | "jx: " -> {img with jx = get_float img.jx entre}
    | "jy: " -> {img with jy = get_float img.jy entre}
    | "dx: " ->
	let v = get_float img.dx entre in
	if v > 0.0 then {img with dx=v}
	else raise (ERREUR(entre,"dimension ngative ou nulle"))
    | "dy: " ->
	let v = get_float img.dy entre in
	if v > 0.0 then {img with dy=v}
	else raise (ERREUR(entre,"dimension ngative ou nulle"))
    | "nmax: " ->
	let v = int_of_string(strip entre) in
	if v > 0 then {img with nmax=v}
	else raise (ERREUR(entre,"nombre ngatif"))
    | _ -> raise(ERREUR(entre,"champ inconnu"))
with
  | Failure f -> raise (ERREUR(entre, "erreur: " ^ f))
  | ERREUR(_) as ex -> raise ex
  | _ -> raise (ERREUR(entre,"erreur indtermine"))


                       (* +--------------------------+
                          |  dition des paramtres  |
                          +--------------------------+ *)

let edit_param data =

  let w,_ = text_size (String.make (en_width+1) 'M') in
  let x0 = data.fdraw.fx + pad in
  let x1 = x0 + pad     in
  let x2 = x1 + w + pad in
  let x3 = x2 + w + pad in
  let y0 = data.fdraw.fy + data.fdraw.fh - bheight in
  let y1 = y0 - bheight   - 4 in
  let y2 = y1 - 6*bheight     in
  let y3 = y2 - bheight - 8   in
  let y4 = data.fdraw.fy + pad + bheight in

  set_color crayon;
  moveto (x1 + 2) (y0+1);
  draw_string "Mandelbrot";
  moveto (x2 + 2) (y0+1);
  draw_string "Julia";
  draw_contour false {fx=x0;   fy=y3;   fw=x3-x0;   fh=y0-y3};
  draw_contour true  {fx=x0-1; fy=y3-1; fw=x3-x0+2; fh=y0-y3+2};

  let mlist = make_entres w bheight x1 y1  ["x0: "; "y0: "; "dx: "; "dy: "; "nmax: "]
  and jlist = (make_entres w bheight x2 y1 ["x0: "; "y0: "; "dx: "; "dy: "; "nmax: "])
	    @ (make_entres w bheight x2 y2 ["jx: "; "jy: "])
  in
  List.iter (set_champ data.mandel) mlist;
  List.iter (set_champ data.julia)  jlist;

  let boutons = make_boutons data.fmsg ["valider"; "annuler"; "quitter"] in
  let ferr = {fx=x0; fy=y4; fw=data.fdraw.fw - 2*pad; fh=bheight} in

  let widgets = { onglets = data.ongl;
		  boutons = boutons;
		  entres = mlist @ jlist} in

  let res = ref(None)
  and e0  = ref(List.hd mlist)
  and err = ref ""
  in

  let validation() = try
    data.mandel <- List.fold_left valide data.mandel mlist;
    data.julia  <- List.fold_left valide data.julia  jlist;
    List.iter (set_champ data.mandel) mlist;
    List.iter (set_champ data.julia)  jlist;
    e0  := List.hd(mlist);
    err := "";
    true
  with ERREUR(entre,msg) ->
    e0  := entre;
    err := msg;
    false
  in

  while !res = None do
    fill_fentre ferr fond_actif;
    set_color crayon;
    moveto (ferr.fx + 2) (ferr.fy+2);
    draw_string !err;
    let r = dite widgets !e0 in
    match r with
      | Some(ONGLET o) ->
	  if o <> (List.hd data.ongl).titre & validation() then res := r
      | Some(BOUTON "quitter") -> res := r;
      | Some(BOUTON "valider") -> let _ = validation() in ()
      | Some(BOUTON "annuler") ->
	  List.iter (set_champ data.mandel) mlist;
	  List.iter (set_champ data.julia)  jlist;
	  err := ""
      | Some(RESIZE) -> res := r;
      | _ -> ()
  done;
  !res

